#!/bin/bash

# this script is a wrapper for BV4627f relay board
# It generally translates plain simple on-off commands to characters what relayboard expects
# also it provides on-off functionality with duration 
# Argo Ellisson 2014, BashPi.org, usage licence: GNU General Public License (GPL) Version 3
# version 1.3

usage() {
echo "USAGE: relay.sh <controller> <relay> <command> <delay>"
echo "WHERE:"
echo "Controller: BV4627 FTDI chip serial number or alias in this script"
echo "Relay: [a-h]|[A-H]"
echo "Command: on|off|onfor|offfor"
echo "Delay: number of seconds to after which to turn on or off relay [0-3600]"
echo "All options are mandatory, including DELAY. Use delay 0 for now."
exit 1
}


if [ $# -eq 4 ];then 

##### LOG #####
DATE=`date`
echo "${DATE} controller:${1} relay:${2}  op:${3} duration:${4} sec"
##### LOG END #####

# controller/board aliases
# add serial numbers below
case "${1}" in
        board1|heating)  BOARD="A1024WGU"
        ;;
        board2|lights)  BOARD="A501W7HV"
        ;;
        board3|access)  BOARD="BOARD3SERIAL"
        ;;
        *) BOARD=${1}
        ;;
esac

SERIALDEVICE="/dev/serialrelay_${BOARD}"
COMMANDFILE="/dev/shm/serialrelay_${BOARD}_commands.lst"

# check does device exist
if [ ! -c ${SERIALDEVICE} ]; then

	echo "Error: selected board is not connected or UDEV rules are not creating symlink ${SERIALDEVICE}. exiting!"
	exit 1

fi

# check does the commandfile exist and is writable - if not, nobody is listening also!
if [ ! -w ${COMMANDFILE} ]; then

        echo "Error: Commandfile does not exist or is not writable. Is service-sript started? exiting!"
        exit 1

fi


# relay selection
# this is a good place for relay aliases also - note "lights" below

case "${2}" in
        a|A|lights)  RELAY="\x41"
        ;;
        b|B)  RELAY="\x42"
        ;;
        c|C)  RELAY="\x43"
        ;;
        d|D)  RELAY="\x44"
        ;;
        e|E)  RELAY="\x45"
        ;;
        f|F)  RELAY="\x46" 
        ;;
        g|G)  RELAY="\x47" 
        ;;
        h|H)  RELAY="\x48"
        ;;
        *) echo "incorrect relay chosen"
        usage
        ;;
esac

# timer config. using it always as its simpler

if [ "${4}" != "0" ];then
        if echo "${4}" | grep -qE ^\-?[0-9]?\.?[0-9]+$; then
                # calculate time
                TIME=$(echo "${4}" |awk '{print int($1*13.6);}')
                if [ ${TIME} -gt 65000 ];then
                        echo "Delay is too big, max ~4700sec/79min/1.3hours"
                        usage
                fi
                if [ ${TIME} -lt 1 ];then
                        TIME=1
                fi
                # convert to hex
                DELAY=`echo -n "${TIME}" |od -t x1 |head -1|awk 'BEGIN{ORS=""}{ for (i=2;i<=NF;i++)print "\x"$i}'`

        else
                echo "Delay is not positive number!"
                usage
        fi

else
        DELAY="\x31"
fi

# Command selection - note that as duration is not supported by relayboard, we send actually 2 commands to relayboard in case of onfor/offfor commands 
# First command in case construct below and second command as USUAL
case "$3" in
        on|ON)  COMM="\x31"
        ;;
        off|OFF) COMM="\x30"
        ;;
        onfor|ONFOR) COMM="\x30"
        # onfor - usual command does timed switch off so first switch that relay on
        echo -en "\x1b\x5b\x31${RELAY}" >> ${COMMANDFILE}
        # note that USUAL command is actually "OFF" above
        ;;
        offfor|OFFFOR)  COMM="\x31"
        # offfor - usual command does timed switch on so first switch that relay off
        echo -en "\x1b\x5b\x30${RELAY}" >> ${COMMANDFILE}
        # note that USUAL command is actually "ON" above
        ;;
        *) echo "incorrect command"
        usage
        ;;
esac

# put USUAL command together / Comma \x2c / ESC \x1b / [ \x5b
COMMAND="\x1b\x5b${COMM}\x2c${DELAY}${RELAY}"

# DEBUG
#echo ${BOARD}
#echo ${RELAY}
#echo ${COMM}
#echo ${DELAY}
#echo ${COMMAND}

# echo command to outputfile
echo -en "${COMMAND}" >> ${COMMANDFILE}

exit 0

fi

usage
